//
//  SSMenuExtra.m
//  SoundSource
//
//  Created by Quentin Carnicelli on 3/23/06.
//  Copyright 2006 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "SSMenuExtra.h"
#import "SSAboutBox.h"
#import "SSAudioDeviceCenter.h"
#import "SSMenuVolumeSliderView.h"
#import "SSPreferences.h"

@implementation SSMenuExtra

- (void)_setupModel
{
	_prefs = [[SSPreferences alloc] init];

	_deviceCenter = [[SSAudioDeviceCenter alloc] init];
	
	if( [_deviceCenter supportsAudioFollowsJack] )
		[_deviceCenter setAudioFollowsJack: [_prefs audioFollowsJack]];
}

+ (NSImage*)_loadImageNamed: (NSString*)name
{
	NSString *path;
	NSImage* image = nil;

	path = [[NSBundle bundleForClass: self] pathForImageResource:name];
	if( path )
		image = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];
	return image;
}

+ (NSImage*)_headphonesImage: (BOOL)highlighted
{
	static NSImage* bgImage = nil;
	static NSImage* bgImageAlt = nil;
	
		if( bgImageAlt == nil )
			bgImageAlt = [[self _loadImageNamed: @"menuIconAlt"] retain];
		if( bgImage == nil )
			bgImage = [[self _loadImageNamed: @"menuIcon"] retain];
	
	return highlighted ? bgImageAlt : bgImage;
}



- (void)_setupView
{
	[self setImage: [[self class] _headphonesImage: NO]];
	[self setAlternateImage: [[self class] _headphonesImage: YES]];
}

- (void)_setupMenu
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle: @"SoundSource menu extra"];
	[menu setDelegate: self];
	[self setMenu: menu];
	[menu release];
}

- (id)initWithBundle:(NSBundle*)bundle
{
	self = [super initWithBundle: bundle];
	
	if( self )
	{
		[self _setupModel];
		[self _setupView];
		[self _setupMenu];
	}
	
	return self;
}

- (void)_safeUnload
{
	[_prefs release];
	_prefs = nil;

	[_deviceCenter release];
	_deviceCenter = nil;
}

- (void)willUnload
{
	[self _safeUnload];
	[super willUnload];
}

- (void)dealloc
{
	[self _safeUnload];
	[super dealloc];
}

- (IBAction)hitOpenAbout: (id)sender
{
	[[SSAboutBox sharedAboutBox] showWindow: self];
}

- (IBAction)hitOpenSoundPrefs: (id)sender
{
	[[NSWorkspace sharedWorkspace] openFile: @"System/Library/PreferencePanes/Sound.prefPane"];
}

- (IBAction)hitOpenAMS: (id)sender
{
	[[NSWorkspace sharedWorkspace]
		launchAppWithBundleIdentifier: @"com.apple.audio.AudioMIDISetup"
		options: NSWorkspaceLaunchDefault
		additionalEventParamDescriptor: nil
		launchIdentifier: nil];
}

- (IBAction)hitSelectInputDevice: (id)sender
{
	SSAudioDevice* device = [sender representedObject];
	if( device && [device isKindOfClass: [SSAudioDevice class]] )
		[_deviceCenter setSelectedInputDevice: device];
}

- (IBAction)hitSelectOutputDevice: (id)sender
{
	SSAudioDevice* device = [sender representedObject];
	if( device && [device isKindOfClass: [SSAudioDevice class]] )
		[_deviceCenter setSelectedOutputDevice: device];
}

- (IBAction)hitSelectSystemDevice: (id)sender
{
	SSAudioDevice* device = [sender representedObject];
	if( device && [device isKindOfClass: [SSAudioDevice class]] )
		[_deviceCenter setSelectedSystemDevice: device];
}

- (IBAction)hitAudioFollowsJack: (id)sender
{
	[_prefs setAudioFollowsJack: ![_prefs audioFollowsJack]];
	[_deviceCenter setAudioFollowsJack: [_prefs audioFollowsJack]];
}

- (void)_menu: (NSMenu*)menu addAudioDevices: (NSArray*)devices selection: (SSAudioDevice*)selectedDevice action: (SEL)action
{
	NSMenuItem* menuItem;

	if( ![devices count] )
	{
		menuItem = (NSMenuItem*)[menu addItemWithTitle: @"<No Devices>" action: nil keyEquivalent: @""];
		[menuItem setIndentationLevel: 1];
	}
	else
	{
		NSEnumerator* deviceEnum;
		SSAudioDevice* device;
		NSString* deviceName;

		devices = [devices sortedArrayUsingSelector: @selector(compare:)];

		deviceEnum = [devices objectEnumerator];
		
		while( (device = [deviceEnum nextObject]) != nil )
		{
			deviceName = [device name];
			if( !deviceName )
				deviceName = @"(Untitled Device)";
			
			menuItem = (NSMenuItem*)[menu addItemWithTitle: deviceName action: action keyEquivalent: @""];
			[menuItem setTarget: self];
			[menuItem setRepresentedObject: device];
			[menuItem setState: [device isEqual: selectedDevice]];
			[menuItem setIndentationLevel: 1];

			if( action == @selector(hitSelectSystemDevice:) ) //:FIXME: Too ugly
				[menuItem setEnabled: [device canBeDefaultSystemDevice]];
			else
				[menuItem setEnabled: [device canBeDefaultDevice]];
		}
	}
}

- (void)_addSliderItemToMenu: (NSMenu*)menu kind: (int)kind
{
	if( floor(NSAppKitVersionNumber) > 824 /*NSAppKitVersionNumber10_4*/ )
	{
		NSMenuItem *item = (NSMenuItem*)[menu addItemWithTitle: @"" action: nil keyEquivalent: @""];
		SSMenuVolumeSliderView *view = [[SSMenuVolumeSliderView alloc] initWithFrame: NSMakeRect( 0, 0, 10, 18 ) deviceCenter: _deviceCenter];
		[view setAutoresizingMask: NSViewWidthSizable];
		[view setKind: kind];
		[(id)item setView: view];
		[view release];
	}
}

- (void)menuNeedsUpdate:(NSMenu*)menu
{
	NSMenuItem* menuItem;
		
	while( [menu numberOfItems] > 0 )
		[menu removeItemAtIndex: 0];
	
	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Output" action: nil keyEquivalent: @""];
	[self _addSliderItemToMenu: menu kind: kSSMenuVolumeOutputKind];
	[self	_menu: menu
			addAudioDevices: [_deviceCenter outputDevices]
			selection: [_deviceCenter selectedOutputDevice]
			action: @selector(hitSelectOutputDevice:)];
	
	[menu addItem: [NSMenuItem separatorItem]];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Input" action: nil keyEquivalent: @""];
	[self _addSliderItemToMenu: menu kind: kSSMenuVolumeInputKind];
	[self	_menu: menu
addAudioDevices: [_deviceCenter inputDevices]
	  selection: [_deviceCenter selectedInputDevice]
		 action: @selector(hitSelectInputDevice:)];
	
	[menu addItem: [NSMenuItem separatorItem]];
	
	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"System" action: nil keyEquivalent: @""];
	[self _addSliderItemToMenu: menu kind: kSSMenuVolumeSystemKind];
	[self	_menu: menu
			addAudioDevices: [_deviceCenter outputDevices]
			selection: [_deviceCenter selectedSystemDevice]
			action: @selector(hitSelectSystemDevice:)];

	if( [_deviceCenter supportsAudioFollowsJack] )
	{
		[menu addItem: [NSMenuItem separatorItem]];
		menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Auto-Switch to Headphones" action: @selector(hitAudioFollowsJack:) keyEquivalent: @""];
		[menuItem setState: [_deviceCenter audioFollowsJack]];
		[menuItem setTarget: self];
	}

	[menu addItem: [NSMenuItem separatorItem]];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Open Sound Preferences..." action: @selector(hitOpenSoundPrefs:) keyEquivalent: @""];
	[menuItem setTarget: self];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Open Audio MIDI Setup..." action: @selector(hitOpenAMS:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menu addItem: [NSMenuItem separatorItem]];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"About SoundSource" action: @selector(hitOpenAbout:) keyEquivalent: @""];
	[menuItem setTarget: self];
}

- (NSString*)AXDescription
{
	return @"SoundSource menu extra";
}

@end
